namespace demo;

struct Date {
    year: uint16;
    month: uint8;
    day: uint8;
}

enum Tag : byte { GEEK, VIP }

table Rating {
    name: string (key, required);
    rating: float;
}

table User {
    name: string;
    authid: string (required);
    uuid: [uint8];
    email: string;
    birthday: Date;
    is_friendly: bool;
    tags: [string];
    ratings: [Rating];
    friends: [uint64];
    referred_by: uint64;
}

root_type User;
